@@ -106,9 +106,13 @@ typedef struct wp_tdmv_bri_ {
 	int		spanno;
 	struct zt_span	span;
 #ifdef DAHDI_ISSUES
+#ifdef DAHDI_22
+	struct dahdi_echocan_state ec[MAX_BRI_LINES];		/* echocan state for each channel */
+#endif
 	struct zt_chan *chans_ptrs[MAX_BRI_LINES];
 #endif
 	struct zt_chan	chans[MAX_BRI_LINES];
+
 	unsigned long	reg_module_map;	/* Registered modules */
 
 	u32		timeslot_map;
@@ -170,6 +174,14 @@ static int wp_tdmv_bri_ec_span(void *pcard);
 
 static void wp_tdmv_bri_dtmf (void* card_id, wan_event_t *event);
 
+#ifdef DAHDI_22
+static int wp_tdmv_bri_hwec_create(struct dahdi_chan *chan, struct dahdi_echocanparams *ecp,
+			   struct dahdi_echocanparam *p, struct dahdi_echocan_state **ec);
+static void wp_tdmv_bri_hwec_free(struct dahdi_chan *chan, struct dahdi_echocan_state *ec);
+#else
+static int wp_bri_zap_hwec(struct zt_chan *chan, int enable);
+#endif
+
 extern int wp_init_proslic(sdla_fe_t *fe, int mod_no, int fast, int sane);
 extern int wp_init_voicedaa(sdla_fe_t *fe, int mod_no, int fast, int sane);
 
@@ -182,6 +194,24 @@ static int wp_tdmv_span_buf_rotate(void *pcard, u32, unsigned long);
 
 static void wp_tdmv_report_alarms(void* pcard, unsigned long te_alarm);
 
+#ifdef DAHDI_22
+/*
+*******************************************************************************
+**			   DAHDI HWEC STRUCTURES
+*******************************************************************************
+*/
+static const struct dahdi_echocan_features wp_tdmv_bri_ec_features = {
+	.NLP_automatic = 1,
+	.CED_tx_detect = 1,
+	.CED_rx_detect = 1,
+};
+
+static const struct dahdi_echocan_ops wp_tdmv_bri_ec_ops = {
+	.name = "WANPIPE_HWEC",
+	.echocan_free = wp_tdmv_bri_hwec_free,
+};
+#endif
+
 /*******************************************************************************
 **			  FUNCTION DEFINITIONS
 *******************************************************************************/
@@ -289,46 +319,135 @@ static int wp_bri_zap_watchdog(struct zt_span *span, int event)
 	return 0;
 }
 
+
+#ifdef DAHDI_22
 /******************************************************************************
-** wp_remora_zap_hwec() - 
+** wp_tdmv_bri_hwec_create() - 
 **
 **	OK
 */
-static int wp_bri_zap_hwec(struct zt_chan *chan, int enable)
+static int wp_tdmv_bri_hwec_create(struct dahdi_chan *chan, struct dahdi_echocanparams *ecp,
+			  struct dahdi_echocanparam *p, struct dahdi_echocan_state **ec)
 {
-	wp_tdmv_bri_t	*wr = NULL;
-	sdla_t		*card = NULL;
-	int		fe_chan = chan->chanpos;
-	int		err = -ENODEV;
-	sdla_fe_t	*fe = NULL;
+	wp_tdmv_bri_t *wr = NULL;
+	sdla_t *card = NULL;
+	int	fe_chan = chan->chanpos;
+	int	err = -ENODEV;
 
-	BRI_FUNC();	
+	if (ecp->param_count > 0) {
+		DEBUG_TDMV("[TDMV] Wanpipe echo canceller does not support parameters; failing request\n");
+		return -EINVAL;
+	}
 
+	BRI_FUNC();
 	WAN_ASSERT2(chan == NULL, -ENODEV);
 	WAN_ASSERT2(chan->pvt == NULL, -ENODEV);
 	wr = chan->pvt;
 	WAN_ASSERT2(wr->card == NULL, -ENODEV);
-	card	= wr->card;
-	fe	= &card->fe;
+	card = wr->card;
 
-	if (card->wandev.ec_enable){
+	*ec = &wr->ec[fe_chan];
+	(*ec)->ops = &wp_tdmv_bri_ec_ops;
+	(*ec)->features = wp_tdmv_bri_ec_features;
+
+	if (card->wandev.ec_enable) {
 		DEBUG_EVENT("[TDMV_BRI]: %s: %s(): channel %d\n",
 			wr->devname, __FUNCTION__, fe_chan);
 
-		if(fe_chan == 1 || fe_chan == 2){
-			err = card->wandev.ec_enable(card, enable, fe_chan);
-		}else{
+		if(fe_chan == 1 || fe_chan == 2) {
+			err = card->wandev.ec_enable(card, 1, fe_chan);
+		} else {
 			DEBUG_EVENT("[TDMV_BRI]: %s: %s(): Warning: invalid fe_channel %d!!\n",
 				wr->devname, __FUNCTION__, fe_chan);
 			err = 0;
 		}
-	}else{
+	} else {
 		DEBUG_EVENT("[TDMV_BRI]: %s: %s(): card->wandev.ec_enable == NULL!!!!!!\n",
 			wr->devname, __FUNCTION__);
 	}
 	return err;
 }
 
+/******************************************************************************
+** wp_tdmv_bri_hwec_free() - 
+**
+**	OK
+*/
+static void wp_tdmv_bri_hwec_free(struct dahdi_chan *chan, struct dahdi_echocan_state *ec)
+{
+	wp_tdmv_bri_t	*wr = NULL;
+	sdla_t *card = NULL;
+	int	fe_chan = chan->chanpos;
+
+	memset(ec, 0, sizeof(*ec));
+
+	if(chan == NULL) return;
+	if(chan->pvt == NULL) return;
+	wr = chan->pvt;
+	if(wr->card == NULL) return;
+	card = wr->card;
+
+	if (card->wandev.ec_enable) {
+		DEBUG_EVENT("[TDMV_BRI]: %s: %s(): channel %d\n",
+			wr->devname, __FUNCTION__, fe_chan);
+
+		if(fe_chan == 1 || fe_chan == 2) {
+			card->wandev.ec_enable(card, 0, fe_chan);
+		} else {
+			DEBUG_EVENT("[TDMV_BRI]: %s: %s(): Warning: invalid fe_channel %d!!\n",
+				wr->devname, __FUNCTION__, fe_chan);
+		}
+	} else {
+		DEBUG_EVENT("[TDMV_BRI]: %s: %s(): card->wandev.ec_enable == NULL!!!!!!\n",
+			wr->devname, __FUNCTION__);
+	}
+}
+
+#else
+
+/******************************************************************************
+** wp_remora_zap_hwec() -
+**
+**  OK
+*/
+static int wp_bri_zap_hwec(struct zt_chan *chan, int enable)
+{
+	wp_tdmv_bri_t   *wr = NULL;
+	sdla_t      *card = NULL;
+	int     fe_chan = chan->chanpos;
+	int     err = -ENODEV;
+	sdla_fe_t   *fe = NULL;
+
+	BRI_FUNC();
+
+	WAN_ASSERT2(chan == NULL, -ENODEV);
+	WAN_ASSERT2(chan->pvt == NULL, -ENODEV);
+	wr = chan->pvt;
+	WAN_ASSERT2(wr->card == NULL, -ENODEV);
+	card    = wr->card;
+	fe  = &card->fe;
+
+	if (card->wandev.ec_enable){
+		DEBUG_EVENT("[TDMV_BRI]: %s: %s(): channel %d\n",
+					wr->devname, __FUNCTION__, fe_chan);
+
+		if(fe_chan == 1 || fe_chan == 2){
+			err = card->wandev.ec_enable(card, enable, fe_chan);
+		}else{
+			DEBUG_EVENT("[TDMV_BRI]: %s: %s(): Warning: invalid fe_channel %d!!\n",
+						wr->devname, __FUNCTION__, fe_chan);
+			err = 0;
+		}
+	}else{
+		DEBUG_EVENT("[TDMV_BRI]: %s: %s(): card->wandev.ec_enable == NULL!!!!!!\n",
+					wr->devname, __FUNCTION__);
+	}
+	return err;
+}
+
+
+#endif
+
 
 /******************************************************************************
 ** wp_tdmv_rbsbits_poll() -
@@ -478,7 +597,11 @@ wr->span.deflaw = ZT_LAW_ALAW;//FIXME: hardcoded
 	wr->span.watchdog	= wp_bri_zap_watchdog;
 	/* Set this pointer only if card has hw echo canceller module */
 	if (card->wandev.ec_dev){
+#ifdef DAHDI_22
+		wr->span.echocan_create = wp_tdmv_bri_hwec_create;
+#else
 		wr->span.echocan = wp_bri_zap_hwec;
+#endif
 	}
 #if defined(__LINUX__)
 	init_waitqueue_head(&wr->span.maintq);
